@@ -26,6 +26,7 @@
 
 #include "nautilus-application.h"
 
+#include <adwaita.h>
 #include <eel/eel-stock-dialogs.h>
 #include <fcntl.h>
 #include <gdk/gdk.h>
@@ -33,7 +34,6 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#include <libadwaita-1/adwaita.h>
 #include <nautilus-extension.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -277,11 +277,7 @@ nautilus_application_create_window (NautilusApplication *self)
 
     if (g_strcmp0 (PROFILE, "") != 0)
     {
-        GtkStyleContext *style_context;
-
-        style_context = gtk_widget_get_style_context (GTK_WIDGET (window));
-
-        gtk_style_context_add_class (style_context, "devel");
+        gtk_widget_add_css_class (GTK_WIDGET (window), "devel");
     }
 
     DEBUG ("Creating a new navigation window");
@@ -583,8 +579,6 @@ nautilus_application_finalize (GObject *object)
     g_clear_object (&priv->progress_handler);
     g_clear_object (&priv->bookmark_list);
 
-    g_clear_object (&priv->fdb_manager);
-
     g_list_free (priv->windows);
 
     g_hash_table_destroy (priv->notifications);
@@ -734,7 +728,6 @@ action_clone_window (GSimpleAction *action,
         {
             location = nautilus_window_slot_get_pending_location (active_slot);
         }
-        g_object_ref (location);
     }
 
     nautilus_application_open_location_full (NAUTILUS_APPLICATION (application), location,
@@ -819,17 +812,6 @@ action_quit (GSimpleAction *action,
     g_list_free (windows);
 }
 
-static void
-action_show_help_overlay (GSimpleAction *action,
-                          GVariant      *state,
-                          gpointer       user_data)
-{
-    GtkApplication *application = user_data;
-    GtkWindow *window = gtk_application_get_active_window (application);
-
-    g_action_group_activate_action (G_ACTION_GROUP (window), "show-help-overlay", NULL);
-}
-
 const static GActionEntry app_entries[] =
 {
     { "new-window", action_new_window, NULL, NULL, NULL },
@@ -839,7 +821,6 @@ const static GActionEntry app_entries[] =
     { "help", action_help, NULL, NULL, NULL },
     { "quit", action_quit, NULL, NULL, NULL },
     { "kill", action_kill, NULL, NULL, NULL },
-    { "show-help-overlay", action_show_help_overlay, NULL, NULL, NULL },
 };
 
 static void
@@ -856,6 +837,8 @@ nautilus_init_application_actions (NautilusApplication *app)
                                           "app.help", "F1");
     nautilus_application_set_accelerator (G_APPLICATION (app),
                                           "app.quit", "<Primary>q");
+    nautilus_application_set_accelerator (G_APPLICATION (app),
+                                          "app.preferences", "<Primary>comma");
 }
 
 static void
@@ -987,7 +970,7 @@ nautilus_application_command_line (GApplication            *application,
 
     if (g_variant_dict_contains (options, "quit"))
     {
-        DEBUG ("Killing application, as requested");
+        DEBUG ("Killing app, as requested");
         g_action_group_activate_action (G_ACTION_GROUP (application),
                                         "kill", NULL);
         goto out;
@@ -1167,9 +1150,10 @@ maybe_migrate_gtk_filechooser_preferences (void)
     }
 }
 
-void
-nautilus_application_startup_common (NautilusApplication *self)
+static void
+nautilus_application_startup (GApplication *app)
 {
+    NautilusApplication *self = NAUTILUS_APPLICATION (app);
     NautilusApplicationPrivate *priv;
 
     nautilus_profile_start (NULL);
@@ -1221,22 +1205,6 @@ nautilus_application_startup_common (NautilusApplication *self)
                              NULL, 0);
 }
 
-static void
-nautilus_application_startup (GApplication *app)
-{
-    NautilusApplication *self = NAUTILUS_APPLICATION (app);
-    NautilusApplicationPrivate *priv;
-
-    nautilus_profile_start (NULL);
-    priv = nautilus_application_get_instance_private (self);
-
-    /* create DBus manager */
-    priv->fdb_manager = nautilus_freedesktop_dbus_new ();
-    nautilus_application_startup_common (self);
-
-    nautilus_profile_end (NULL);
-}
-
 static gboolean
 nautilus_application_dbus_register (GApplication     *app,
                                     GDBusConnection  *connection,
@@ -1253,6 +1221,12 @@ nautilus_application_dbus_register (GApplication     *app,
         return FALSE;
     }
 
+    priv->fdb_manager = nautilus_freedesktop_dbus_new (connection);
+    if (!nautilus_freedesktop_dbus_register (priv->fdb_manager, error))
+    {
+        return FALSE;
+    }
+
     priv->search_provider = nautilus_shell_search_provider_new ();
     if (!nautilus_shell_search_provider_register (priv->search_provider, connection, error))
     {
@@ -1279,6 +1253,12 @@ nautilus_application_dbus_unregister (GApplication    *app,
         g_clear_object (&priv->dbus_manager);
     }
 
+    if (priv->fdb_manager)
+    {
+        nautilus_freedesktop_dbus_unregister (priv->fdb_manager);
+        g_clear_object (&priv->fdb_manager);
+    }
+
     if (priv->search_provider)
     {
         nautilus_shell_search_provider_unregister (priv->search_provider);
@@ -1312,14 +1292,6 @@ update_dbus_opened_locations (NautilusApplication *self)
     g_return_if_fail (NAUTILUS_IS_APPLICATION (self));
 
     priv = nautilus_application_get_instance_private (self);
-
-    /* Children of nautilus application could not handle the dbus, so don't
-     * do anything in that case */
-    if (!priv->fdb_manager)
-    {
-        return;
-    }
-
     dbus_object_path = g_application_get_dbus_object_path (G_APPLICATION (self));
 
     g_return_if_fail (dbus_object_path);
