@@ -445,7 +445,6 @@ listbox_header_func (GtkListBoxRow             *row,
     if (separator == NULL)
     {
         separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-        gtk_widget_show (separator);
 
         gtk_list_box_row_set_header (row, separator);
     }
@@ -534,8 +533,6 @@ create_original_name_label (NautilusBatchRenameDialog *dialog,
 
     dialog->listbox_labels_old = g_list_prepend (dialog->listbox_labels_old, label_old);
 
-    gtk_widget_show (label_old);
-
     return label_old;
 }
 
@@ -553,8 +550,6 @@ create_result_label (NautilusBatchRenameDialog *dialog,
 
     dialog->listbox_labels_new = g_list_prepend (dialog->listbox_labels_new, label_new);
 
-    gtk_widget_show (label_new);
-
     return label_new;
 }
 
@@ -579,8 +574,6 @@ create_arrow (NautilusBatchRenameDialog *dialog,
 
     dialog->listbox_icons = g_list_prepend (dialog->listbox_icons, icon);
 
-    gtk_widget_show (icon);
-
     return icon;
 }
 
@@ -604,7 +597,6 @@ prepare_batch_rename (NautilusBatchRenameDialog *dialog)
 
     gtk_widget_set_cursor_from_name (GTK_WIDGET (dialog), "progress");
 
-    gtk_widget_hide (GTK_WIDGET (dialog));
     begin_batch_rename (dialog, dialog->new_names);
 
     gtk_window_destroy (GTK_WINDOW (dialog));
@@ -787,7 +779,6 @@ update_conflict_row_background (NautilusBatchRenameDialog *dialog)
     GList *l3;
     GList *duplicates;
     gint index;
-    GtkStyleContext *context;
     ConflictData *conflict_data;
 
     index = 0;
@@ -804,17 +795,11 @@ update_conflict_row_background (NautilusBatchRenameDialog *dialog)
         GtkWidget *row2 = gtk_widget_get_parent (l2->data);
         GtkWidget *row3 = gtk_widget_get_parent (l3->data);
 
-        context = gtk_widget_get_style_context (row1);
-
-        if (gtk_style_context_has_class (context, "conflict-row"))
+        if (gtk_widget_has_css_class (row1, "conflict-row"))
         {
-            gtk_style_context_remove_class (context, "conflict-row");
-
-            context = gtk_widget_get_style_context (row2);
-            gtk_style_context_remove_class (context, "conflict-row");
-
-            context = gtk_widget_get_style_context (row3);
-            gtk_style_context_remove_class (context, "conflict-row");
+            gtk_widget_remove_css_class (row1, "conflict-row");
+            gtk_widget_remove_css_class (row2, "conflict-row");
+            gtk_widget_remove_css_class (row3, "conflict-row");
         }
 
         if (duplicates != NULL)
@@ -822,14 +807,9 @@ update_conflict_row_background (NautilusBatchRenameDialog *dialog)
             conflict_data = duplicates->data;
             if (conflict_data->index == index)
             {
-                context = gtk_widget_get_style_context (row1);
-                gtk_style_context_add_class (context, "conflict-row");
-
-                context = gtk_widget_get_style_context (row2);
-                gtk_style_context_add_class (context, "conflict-row");
-
-                context = gtk_widget_get_style_context (row3);
-                gtk_style_context_add_class (context, "conflict-row");
+                gtk_widget_add_css_class (row1, "conflict-row");
+                gtk_widget_add_css_class (row2, "conflict-row");
+                gtk_widget_add_css_class (row3, "conflict-row");
 
                 duplicates = duplicates->next;
             }
@@ -903,7 +883,7 @@ update_listbox (NautilusBatchRenameDialog *dialog)
 
         gtk_widget_set_sensitive (dialog->rename_button, FALSE);
 
-        gtk_widget_show (dialog->conflict_box);
+        gtk_widget_set_visible (dialog->conflict_box, TRUE);
 
         dialog->selected_conflict = 0;
         dialog->conflicts_number = g_list_length (dialog->duplicates);
@@ -923,7 +903,7 @@ update_listbox (NautilusBatchRenameDialog *dialog)
     }
     else
     {
-        gtk_widget_hide (dialog->conflict_box);
+        gtk_widget_set_visible (dialog->conflict_box, FALSE);
 
         /* re-enable the rename button if there are no more name conflicts */
         if (dialog->duplicates == NULL && !gtk_widget_is_sensitive (dialog->rename_button))
@@ -1251,13 +1231,13 @@ have_unallowed_character (NautilusBatchRenameDialog *dialog)
         gtk_widget_set_sensitive (dialog->conflict_down, FALSE);
         gtk_widget_set_sensitive (dialog->conflict_up, FALSE);
 
-        gtk_widget_show (dialog->conflict_box);
+        gtk_widget_set_visible (dialog->conflict_box, TRUE);
 
         return TRUE;
     }
     else
     {
-        gtk_widget_hide (dialog->conflict_box);
+        gtk_widget_set_visible (dialog->conflict_box, FALSE);
 
         return FALSE;
     }
@@ -1416,7 +1396,6 @@ static void
 update_row_shadowing (GtkWidget *row,
                       gboolean   shown)
 {
-    GtkStyleContext *context;
     GtkStateFlags flags;
 
     if (!GTK_IS_LIST_BOX_ROW (row))
@@ -1424,8 +1403,7 @@ update_row_shadowing (GtkWidget *row,
         return;
     }
 
-    context = gtk_widget_get_style_context (row);
-    flags = gtk_style_context_get_state (context);
+    flags = gtk_widget_get_state_flags (row);
 
     if (shown)
     {
@@ -1436,7 +1414,7 @@ update_row_shadowing (GtkWidget *row,
         flags &= ~GTK_STATE_FLAG_PRELIGHT;
     }
 
-    gtk_style_context_set_state (context, flags);
+    gtk_widget_set_state_flags (row, flags, FALSE);
 }
 
 static void
@@ -1606,11 +1584,9 @@ get_tags_intersecting_sorted (NautilusBatchRenameDialog *self,
             if (text_changed_mode == TEXT_WAS_DELETED)
             {
                 selection_intersects_tag_start = end_position > tag_data->position &&
-                                                 end_position <= tag_end_position &&
-                                                 start_position <= tag_data->position;
+                                                 end_position <= tag_end_position;
                 selection_intersects_tag_end = start_position >= tag_data->position &&
-                                               start_position < tag_end_position &&
-                                               end_position >= tag_end_position;
+                                               start_position < tag_end_position;
                 tag_is_contained_in_selection = start_position <= tag_data->position &&
                                                 end_position >= tag_end_position;
             }
