@@ -32,7 +32,6 @@
 #include "nautilus-search-directory.h"
 #include "nautilus-starred-directory.h"
 #include "nautilus-ui-utilities.h"
-#include <eel/eel-glib-extensions.h>
 #include <eel/eel-string.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-vfs-extensions.h>
@@ -1418,26 +1417,6 @@ nautilus_file_can_rename_files (GList *files)
     return TRUE;
 }
 
-/* Try to get a native file:// URI instead of any other GVFS
- * scheme, for interoperability with apps only handling file:// URIs.
- */
-gchar *
-nautilus_uri_to_native_uri (const gchar *uri)
-{
-    g_autoptr (GFile) file = NULL;
-    g_autofree gchar *path = NULL;
-
-    file = g_file_new_for_uri (uri);
-    path = g_file_get_path (file);
-
-    if (path != NULL)
-    {
-        return g_filename_to_uri (path, NULL, NULL);
-    }
-
-    return NULL;
-}
-
 NautilusQueryRecursive
 location_settings_search_get_recursive (void)
 {
