@@ -258,6 +258,8 @@ gboolean            nautilus_files_view_should_show_file                 (Nautil
 gboolean            nautilus_files_view_should_sort_directories_first    (NautilusFilesView *view);
 void                nautilus_files_view_ignore_hidden_file_preferences   (NautilusFilesView *view);
 
+gboolean            nautilus_files_view_has_subdirectory                (NautilusFilesView *view,
+                                                                         NautilusDirectory *directory);
 void                nautilus_files_view_add_subdirectory                (NautilusFilesView *view,
                                                                          NautilusDirectory *directory);
 void                nautilus_files_view_remove_subdirectory             (NautilusFilesView *view,
@@ -276,16 +278,7 @@ void              nautilus_files_view_new_file_with_initial_contents (NautilusFi
                                                                       const char         *parent_uri,
                                                                       const char         *filename,
                                                                       const char         *initial_contents,
-                                                                      int                 length);
-
-/* clipboard reading */
-void               nautilus_files_view_get_clipboard_async  (NautilusFilesView   *self,
-                                                             GAsyncReadyCallback  callback,
-                                                             gpointer             callback_data);
-NautilusClipboard *nautilus_files_view_get_clipboard_finish (NautilusFilesView  *self,
-                                                             GAsyncResult       *result,
-                                                             GError            **error);
-
+                                                                      gsize               length);
 /* selection handling */
 void              nautilus_files_view_activate_selection         (NautilusFilesView      *view);
 void              nautilus_files_view_preview_selection_event    (NautilusFilesView      *view,
