@@ -31,7 +31,6 @@ update_icon (NautilusGridCell *self)
     g_autoptr (NautilusViewItem) item = NULL;
     NautilusFileIconFlags flags;
     g_autoptr (GdkPaintable) icon_paintable = NULL;
-    GtkStyleContext *style_context;
     NautilusFile *file;
     guint icon_size;
     gint scale_factor;
@@ -50,16 +49,15 @@ update_icon (NautilusGridCell *self)
     /* Set the same height and width for all icons regardless of aspect ratio.
      */
     gtk_widget_set_size_request (self->fixed_height_box, icon_size, icon_size);
-    style_context = gtk_widget_get_style_context (self->icon);
     thumbnail_path = nautilus_file_get_thumbnail_path (file);
     if (thumbnail_path != NULL &&
         nautilus_file_should_show_thumbnail (file))
     {
-        gtk_style_context_add_class (style_context, "thumbnail");
+        gtk_widget_add_css_class (self->icon, "thumbnail");
     }
     else
     {
-        gtk_style_context_remove_class (style_context, "thumbnail");
+        gtk_widget_remove_css_class (self->icon, "thumbnail");
     }
 }
 
@@ -118,13 +116,7 @@ update_emblems (NautilusGridCell *self)
     emblems = nautilus_file_get_emblem_icons (file);
     for (GList *l = emblems; l != NULL; l = l->next)
     {
-        g_autoptr (GtkIconPaintable) icon_paintable = NULL;
-
-        /* Workaround for gtk_icon_theme_has_gicon() ignoring unthemed icons.
-         * See: https://gitlab.gnome.org/GNOME/nautilus/-/issues/2789 and
-         * https://gitlab.gnome.org/GNOME/gtk/-/issues/5709 */
-        icon_paintable = gtk_icon_theme_lookup_by_gicon (theme, l->data, 16, 1, 0, 0);
-        if (g_strcmp0 (gtk_icon_paintable_get_icon_name (icon_paintable), "image-missing") == 0)
+        if (!gtk_icon_theme_has_gicon (theme, l->data))
         {
             g_autofree gchar *icon_string = g_icon_to_string (l->data);
             g_warning ("Failed to add emblem. “%s” not found in the icon theme",
