@@ -56,155 +56,123 @@ struct _NautilusListBasePrivate
     graphene_point_t hover_start_point;
     guint hover_timer_id;
     GtkDropTarget *view_drop_target;
+
+    GCancellable *clipboard_cancellable;
 };
 
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (NautilusListBase, nautilus_list_base, NAUTILUS_TYPE_FILES_VIEW)
 
-typedef struct
+static const char *
+get_sort_attribute_from_sort_type (NautilusFileSortType sort_type)
 {
-    const NautilusFileSortType sort_type;
-    const gchar *metadata_name;
-} SortConstants;
+    switch (sort_type)
+    {
+        case NAUTILUS_FILE_SORT_BY_DISPLAY_NAME:
+        {
+            return "name";
+        }
 
-static const SortConstants sorts_constants[] =
-{
-    {
-        NAUTILUS_FILE_SORT_BY_DISPLAY_NAME,
-        "name",
-    },
-    {
-        NAUTILUS_FILE_SORT_BY_SIZE,
-        "size",
-    },
-    {
-        NAUTILUS_FILE_SORT_BY_TYPE,
-        "type",
-    },
-    {
-        NAUTILUS_FILE_SORT_BY_MTIME,
-        "date_modified",
-    },
-    {
-        NAUTILUS_FILE_SORT_BY_ATIME,
-        "date_accessed",
-    },
-    {
-        NAUTILUS_FILE_SORT_BY_BTIME,
-        "date_created",
-    },
-    {
-        NAUTILUS_FILE_SORT_BY_TRASHED_TIME,
-        "trashed_on",
-    },
-    {
-        NAUTILUS_FILE_SORT_BY_SEARCH_RELEVANCE,
-        "search_relevance",
-    },
-    {
-        NAUTILUS_FILE_SORT_BY_RECENCY,
-        "recency",
-    },
-};
+        case NAUTILUS_FILE_SORT_BY_SIZE:
+        {
+            return "size";
+        }
+
+        case NAUTILUS_FILE_SORT_BY_TYPE:
+        {
+            return "type";
+        }
+
+        case NAUTILUS_FILE_SORT_BY_MTIME:
+        {
+            return "date_modified";
+        }
+
+        case NAUTILUS_FILE_SORT_BY_ATIME:
+        {
+            return "date_accessed";
+        }
+
+        case NAUTILUS_FILE_SORT_BY_BTIME:
+        {
+            return "date_created";
+        }
+
+        case NAUTILUS_FILE_SORT_BY_TRASHED_TIME:
+        {
+            return "trashed_on";
+        }
+
+        case NAUTILUS_FILE_SORT_BY_SEARCH_RELEVANCE:
+        {
+            return "search_relevance";
+        }
+
+        case NAUTILUS_FILE_SORT_BY_RECENCY:
+        {
+            return "recency";
+        }
+
+        case NAUTILUS_FILE_SORT_BY_STARRED:
+        {
+            return "starred";
+        }
+    }
+
+    return NULL;
+}
 
 static inline NautilusViewItem *
 get_view_item (GListModel *model,
                guint       position)
 {
-    return NAUTILUS_VIEW_ITEM (g_list_model_get_item (model, position));
+    g_autoptr (GtkTreeListRow) row = g_list_model_get_item (model, position);
+
+    g_return_val_if_fail (GTK_IS_TREE_LIST_ROW (row), NULL);
+    return NAUTILUS_VIEW_ITEM (gtk_tree_list_row_get_item (GTK_TREE_LIST_ROW (row)));
 }
 
-static const SortConstants *
-get_sorts_constants_from_sort_type (NautilusFileSortType sort_type)
-{
-    guint i;
-
-    for (i = 0; i < G_N_ELEMENTS (sorts_constants); i++)
-    {
-        if (sort_type == sorts_constants[i].sort_type)
-        {
-            return &sorts_constants[i];
-        }
-    }
-
-    return &sorts_constants[0];
-}
-
-static const SortConstants *
-get_sorts_constants_from_metadata_text (const char *metadata_name)
-{
-    guint i;
-
-    for (i = 0; i < G_N_ELEMENTS (sorts_constants); i++)
-    {
-        if (g_strcmp0 (sorts_constants[i].metadata_name, metadata_name) == 0)
-        {
-            return &sorts_constants[i];
-        }
-    }
-
-    return &sorts_constants[0];
-}
-
-const NautilusFileSortType
-get_sorts_type_from_metadata_text (const char *metadata_name)
-{
-    return get_sorts_constants_from_metadata_text (metadata_name)->sort_type;
-}
-
-static const SortConstants *
-get_default_sort_order (NautilusFile *file,
-                        gboolean     *reversed)
-{
-    NautilusFileSortType sort_type;
-
-    sort_type = nautilus_file_get_default_sort_type (file, reversed);
-
-    return get_sorts_constants_from_sort_type (sort_type);
-}
-
-static const SortConstants *
+static char *
 get_directory_sort_by (NautilusFile *file,
                        gboolean     *reversed)
 {
-    const SortConstants *default_sort;
-    g_autofree char *sort_by = NULL;
+    NautilusFileSortType default_sort;
+    char *sort_by = NULL;
 
-    default_sort = get_default_sort_order (file, reversed);
-    g_return_val_if_fail (default_sort != NULL, NULL);
+    default_sort = nautilus_file_get_default_sort_type (file, reversed);
 
-    if (default_sort->sort_type == NAUTILUS_FILE_SORT_BY_RECENCY ||
-        default_sort->sort_type == NAUTILUS_FILE_SORT_BY_TRASHED_TIME ||
-        default_sort->sort_type == NAUTILUS_FILE_SORT_BY_SEARCH_RELEVANCE)
+    if (default_sort == NAUTILUS_FILE_SORT_BY_RECENCY ||
+        default_sort == NAUTILUS_FILE_SORT_BY_TRASHED_TIME ||
+        default_sort == NAUTILUS_FILE_SORT_BY_SEARCH_RELEVANCE)
     {
         /* These defaults are important. Ignore metadata. */
-        return default_sort;
+        return g_strdup (get_sort_attribute_from_sort_type (default_sort));
     }
 
     sort_by = nautilus_file_get_metadata (file,
                                           NAUTILUS_METADATA_KEY_ICON_VIEW_SORT_BY,
-                                          default_sort->metadata_name);
+                                          get_sort_attribute_from_sort_type (default_sort));
 
     *reversed = nautilus_file_get_boolean_metadata (file,
                                                     NAUTILUS_METADATA_KEY_ICON_VIEW_SORT_REVERSED,
                                                     *reversed);
 
-    return get_sorts_constants_from_metadata_text (sort_by);
+    return sort_by;
 }
 
 void
 set_directory_sort_metadata (NautilusFile *file,
-                             const gchar  *metadata_name,
+                             const gchar  *sort_attribute,
                              gboolean      reversed)
 {
-    const SortConstants *default_sort;
+    NautilusFileSortType default_sort;
     gboolean default_reversed;
 
-    default_sort = get_default_sort_order (file, &default_reversed);
+    default_sort = nautilus_file_get_default_sort_type (file, &default_reversed);
 
     nautilus_file_set_metadata (file,
                                 NAUTILUS_METADATA_KEY_ICON_VIEW_SORT_BY,
-                                default_sort->metadata_name,
-                                metadata_name);
+                                get_sort_attribute_from_sort_type (default_sort),
+                                sort_attribute);
     nautilus_file_set_boolean_metadata (file,
                                         NAUTILUS_METADATA_KEY_ICON_VIEW_SORT_REVERSED,
                                         default_reversed,
@@ -214,17 +182,17 @@ set_directory_sort_metadata (NautilusFile *file,
 static void
 update_sort_order_from_metadata_and_preferences (NautilusListBase *self)
 {
-    const SortConstants *default_directory_sort;
+    g_autofree char *sort_attribute = NULL;
     GActionGroup *view_action_group;
     gboolean reversed;
 
-    default_directory_sort = get_directory_sort_by (nautilus_files_view_get_directory_as_file (NAUTILUS_FILES_VIEW (self)),
-                                                    &reversed);
+    sort_attribute = get_directory_sort_by (nautilus_files_view_get_directory_as_file (NAUTILUS_FILES_VIEW (self)),
+                                            &reversed);
     view_action_group = nautilus_files_view_get_action_group (NAUTILUS_FILES_VIEW (self));
     g_action_group_change_action_state (view_action_group,
                                         "sort",
                                         g_variant_new ("(sb)",
-                                                       default_directory_sort->metadata_name,
+                                                       sort_attribute,
                                                        reversed));
 }
 
@@ -247,7 +215,7 @@ nautilus_list_base_set_icon_size (NautilusListBase *self,
     }
 }
 
-static void
+void
 set_focus_item (NautilusListBase *self,
                 NautilusViewItem *item)
 {
@@ -349,9 +317,9 @@ rubberband_set_state (NautilusListBase *self,
                       gboolean          enabled)
 {
     /* This is a temporary workaround to deal with the rubberbanding issues
-     * during a drag and drop.  Disable rubberband on item press and enable
-     * rubberbnad on item release/stop.  The permanent solution for this is
-     * https://gitlab.gnome.org/GNOME/gtk/-/merge_requests/4831 */
+     * during a drag and drop. Disable rubberband on item press and enable
+     * rubberband on item release/stop. See:
+     * https://gitlab.gnome.org/GNOME/gtk/-/issues/5670 */
 
     GtkWidget *view;
 
@@ -718,48 +686,6 @@ get_preferred_action (NautilusFile *target_file,
     return action;
 }
 
-static void
-real_perform_drop (NautilusListBase *self,
-                   const GValue     *value,
-                   GdkDragAction     action,
-                   GFile            *target_location)
-{
-    g_autofree gchar *target_uri = g_file_get_uri (target_location);
-
-    if (!gdk_drag_action_is_unique (action))
-    {
-        /* TODO: Implement */
-    }
-    else if (G_VALUE_HOLDS (value, G_TYPE_STRING))
-    {
-        nautilus_files_view_handle_text_drop (NAUTILUS_FILES_VIEW (self),
-                                              g_value_get_string (value),
-                                              target_uri, action);
-    }
-    else if (G_VALUE_HOLDS (value, GDK_TYPE_FILE_LIST))
-    {
-        GSList *source_file_list = g_value_get_boxed (value);
-        GList *source_uri_list = NULL;
-
-        if (source_file_list == NULL)
-        {
-            return;
-        }
-
-        for (GSList *l = source_file_list; l != NULL; l = l->next)
-        {
-            source_uri_list = g_list_prepend (source_uri_list, g_file_get_uri (l->data));
-        }
-        source_uri_list = g_list_reverse (source_uri_list);
-
-        nautilus_files_view_drop_proxy_received_uris (NAUTILUS_FILES_VIEW (self),
-                                                      source_uri_list,
-                                                      target_uri,
-                                                      action);
-        g_list_free_full (source_uri_list, g_free);
-    }
-}
-
 static GdkDragAction
 on_item_drag_enter (GtkDropTarget *target,
                     double         x,
@@ -885,9 +811,7 @@ on_item_drop (GtkDropTarget *target,
     /* In x11 the leave signal isn't emitted on a drop so we need to clear the timeout */
     g_clear_handle_id (&priv->hover_timer_id, g_source_remove);
 
-    real_perform_drop (self, value, actions, target_location);
-
-    return TRUE;
+    return nautilus_dnd_perform_drop (NAUTILUS_FILES_VIEW (self), value, actions, target_location);
 }
 
 static GdkDragAction
@@ -980,21 +904,21 @@ on_view_drop (GtkDropTarget *target,
     }
     #endif
 
-    real_perform_drop (self, value, actions, target_location);
-
-    return TRUE;
+    return nautilus_dnd_perform_drop (NAUTILUS_FILES_VIEW (self), value, actions, target_location);
 }
 
 void
 setup_cell_common (GtkListItem      *listitem,
                    NautilusViewCell *cell)
 {
+    GtkExpression *expression;
     GtkEventController *controller;
     GtkDropTarget *drop_target;
 
-    g_object_bind_property (listitem, "item",
-                            cell, "item",
-                            G_BINDING_SYNC_CREATE);
+    expression = gtk_property_expression_new (GTK_TYPE_LIST_ITEM, NULL, "item");
+    expression = gtk_property_expression_new (GTK_TYPE_TREE_LIST_ROW, expression, "item");
+    gtk_expression_bind (expression, cell, "item", listitem);
+
     gtk_list_item_set_child (listitem, GTK_WIDGET (cell));
 
     controller = GTK_EVENT_CONTROLLER (gtk_gesture_click_new ());
@@ -1027,14 +951,34 @@ setup_cell_common (GtkListItem      *listitem,
     g_signal_connect (drop_target, "motion", G_CALLBACK (on_item_drag_motion), cell);
     g_signal_connect (drop_target, "drop", G_CALLBACK (on_item_drop), cell);
     gtk_widget_add_controller (GTK_WIDGET (cell), GTK_EVENT_CONTROLLER (drop_target));
+}
 
-    controller = gtk_drop_controller_motion_new ();
-    gtk_widget_add_controller (GTK_WIDGET (cell), controller);
+static void
+real_setup_cell_hover (NautilusViewCell *cell,
+                       GtkWidget        *target)
+{
+    GtkEventController *controller = gtk_drop_controller_motion_new ();
+    gtk_widget_add_controller (target, controller);
     g_signal_connect (controller, "enter", G_CALLBACK (on_item_drag_hover_enter), cell);
     g_signal_connect (controller, "leave", G_CALLBACK (on_item_drag_hover_leave), cell);
     g_signal_connect (controller, "motion", G_CALLBACK (on_item_drag_hover_motion), cell);
 }
 
+void
+setup_cell_hover_inner_target (NautilusViewCell *cell,
+                               GtkWidget        *target)
+{
+    g_return_if_fail (gtk_widget_is_ancestor (target, GTK_WIDGET (cell)));
+
+    real_setup_cell_hover (cell, target);
+}
+
+void
+setup_cell_hover (NautilusViewCell *cell)
+{
+    real_setup_cell_hover (cell, GTK_WIDGET (cell));
+}
+
 static void
 nautilus_list_base_scroll_to_item (NautilusListBase *self,
                                    guint             position)
@@ -1132,7 +1076,8 @@ real_file_changed (NautilusFilesView *files_view,
 }
 
 static GList *
-real_get_selection (NautilusFilesView *files_view)
+get_selection (NautilusFilesView *files_view,
+               gboolean           for_file_transfer)
 {
     NautilusListBase *self = NAUTILUS_LIST_BASE (files_view);
     NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
@@ -1145,10 +1090,26 @@ real_get_selection (NautilusFilesView *files_view)
     for (guint i = 0; i < n_selected; i++)
     {
         g_autoptr (NautilusViewItem) item = NULL;
+        NautilusFile *file;
 
         item = get_view_item (G_LIST_MODEL (selection), i);
-        selected_files = g_list_prepend (selected_files,
-                                         g_object_ref (nautilus_view_item_get_file (item)));
+        file = nautilus_view_item_get_file (item);
+
+        if (for_file_transfer)
+        {
+            /* If the parent is already selected don't include the child. */
+            g_autoptr (NautilusFile) parent = nautilus_file_get_parent (file);
+            NautilusViewItem *parent_item;
+            guint parent_pos;
+
+            parent_item = nautilus_view_model_get_item_from_file (priv->model, parent);
+            parent_pos = nautilus_view_model_get_index (priv->model, parent_item);
+            if (gtk_selection_model_is_selected (GTK_SELECTION_MODEL (priv->model), parent_pos))
+            {
+                continue;
+            }
+        }
+        selected_files = g_list_prepend (selected_files, g_object_ref (file));
     }
 
     selected_files = g_list_reverse (selected_files);
@@ -1156,6 +1117,18 @@ real_get_selection (NautilusFilesView *files_view)
     return selected_files;
 }
 
+static GList *
+real_get_selection (NautilusFilesView *files_view)
+{
+    return get_selection (files_view, FALSE);
+}
+
+static GList *
+real_get_selection_for_file_transfer (NautilusFilesView *files_view)
+{
+    return get_selection (files_view, TRUE);
+}
+
 static gboolean
 real_is_empty (NautilusFilesView *files_view)
 {
@@ -1169,8 +1142,9 @@ static void
 real_end_file_changes (NautilusFilesView *files_view)
 {
     NautilusListBase *self = NAUTILUS_LIST_BASE (files_view);
+    NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
 
-    nautilus_list_base_reset_sort (self);
+    nautilus_view_model_sort (priv->model);
 }
 
 static void
@@ -1185,7 +1159,7 @@ real_remove_file (NautilusFilesView *files_view,
     item = nautilus_view_model_get_item_from_file (priv->model, file);
     if (item != NULL)
     {
-        nautilus_view_model_remove_item (priv->model, item);
+        nautilus_view_model_remove_item (priv->model, item, directory);
         nautilus_files_view_notify_selection_changed (files_view);
     }
 }
@@ -1328,11 +1302,14 @@ on_clipboard_contents_received (GObject      *source_object,
                                 GAsyncResult *res,
                                 gpointer      user_data)
 {
-    NautilusFilesView *files_view = NAUTILUS_FILES_VIEW (source_object);
+    NautilusFilesView *files_view = NAUTILUS_FILES_VIEW (user_data);
     NautilusListBase *self = NAUTILUS_LIST_BASE (files_view);
     NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
     NautilusClipboard *clip;
     NautilusViewItem *item;
+    const GValue *value;
+
+    value = gdk_clipboard_read_value_finish (GDK_CLIPBOARD (source_object), res, NULL);
 
     for (GList *l = priv->cut_files; l != NULL; l = l->next)
     {
@@ -1344,7 +1321,15 @@ on_clipboard_contents_received (GObject      *source_object,
     }
     g_clear_list (&priv->cut_files, g_object_unref);
 
-    clip = nautilus_files_view_get_clipboard_finish (files_view, res, NULL);
+    if (value != NULL && G_VALUE_HOLDS (value, NAUTILUS_TYPE_CLIPBOARD))
+    {
+        clip = g_value_get_boxed (value);
+    }
+    else
+    {
+        return;
+    }
+
     if (clip != NULL && nautilus_clipboard_is_cut (clip))
     {
         priv->cut_files = g_list_copy_deep (nautilus_clipboard_peek_files (clip),
@@ -1365,9 +1350,22 @@ on_clipboard_contents_received (GObject      *source_object,
 static void
 update_clipboard_status (NautilusFilesView *view)
 {
-    nautilus_files_view_get_clipboard_async (view,
-                                             on_clipboard_contents_received,
-                                             NULL);
+    NautilusListBase *self = NAUTILUS_LIST_BASE (view);
+    NautilusListBasePrivate *priv = nautilus_list_base_get_instance_private (self);
+    GdkClipboard *clipboard;
+    GdkContentFormats *formats;
+
+    clipboard = gtk_widget_get_clipboard (GTK_WIDGET (view));
+    formats = gdk_clipboard_get_formats (clipboard);
+
+    if (gdk_content_formats_contain_gtype (formats, NAUTILUS_TYPE_CLIPBOARD))
+    {
+        gdk_clipboard_read_value_async (clipboard, NAUTILUS_TYPE_CLIPBOARD,
+                                        G_PRIORITY_DEFAULT,
+                                        priv->clipboard_cancellable,
+                                        on_clipboard_contents_received,
+                                        view);
+    }
 }
 
 static void
@@ -1639,6 +1637,9 @@ nautilus_list_base_dispose (GObject *object)
     g_clear_handle_id (&priv->prioritize_thumbnailing_handle_id, g_source_remove);
     g_clear_handle_id (&priv->hover_timer_id, g_source_remove);
 
+    g_cancellable_cancel (priv->clipboard_cancellable);
+    g_clear_object (&priv->clipboard_cancellable);
+
     G_OBJECT_CLASS (nautilus_list_base_parent_class)->dispose (object);
 }
 
@@ -1798,10 +1799,7 @@ nautilus_list_base_class_init (NautilusListBaseClass *klass)
     files_view_class->click_policy_changed = real_click_policy_changed;
     files_view_class->file_changed = real_file_changed;
     files_view_class->get_selection = real_get_selection;
-    /* TODO: remove this get_selection_for_file_transfer, this doesn't even
-     * take into account we could us the view for recursive search :/
-     * CanvasView has the same issue. */
-    files_view_class->get_selection_for_file_transfer = real_get_selection;
+    files_view_class->get_selection_for_file_transfer = real_get_selection_for_file_transfer;
     files_view_class->is_empty = real_is_empty;
     files_view_class->remove_file = real_remove_file;
     files_view_class->select_all = real_select_all;
@@ -1859,6 +1857,8 @@ nautilus_list_base_init (NautilusListBase *self)
                              G_CONNECT_SWAPPED);
 
     set_click_mode_from_settings (self);
+
+    priv->clipboard_cancellable = g_cancellable_new ();
 }
 
 NautilusViewModel *
@@ -1900,13 +1900,3 @@ nautilus_list_base_setup_gestures (NautilusListBase *self)
     gtk_widget_add_controller (GTK_WIDGET (self), GTK_EVENT_CONTROLLER (drop_target));
     priv->view_drop_target = drop_target;
 }
-
-void
-nautilus_list_base_reset_sort (NautilusListBase *self)
-{
-    NautilusViewModel *model;
-
-    /* Reset the sorter to trigger ressorting */
-    model = nautilus_list_base_get_model (NAUTILUS_LIST_BASE (self));
-    nautilus_view_model_set_sorter (model, nautilus_view_model_get_sorter (model));
-}
