@@ -27,6 +27,9 @@
 #include "nautilus-star-cell.h"
 #include "nautilus-tag-manager.h"
 
+/* We wait two seconds after row is collapsed to unload the subdirectory */
+#define COLLAPSE_TO_UNLOAD_DELAY 2
+
 struct _NautilusListView
 {
     NautilusListBase parent_instance;
@@ -37,6 +40,7 @@ struct _NautilusListView
     gint zoom_level;
 
     gboolean directories_first;
+    gboolean expand_as_a_tree;
 
     GQuark path_attribute_q;
     GFile *file_path_base_location;
@@ -44,21 +48,11 @@ struct _NautilusListView
     GtkColumnViewColumn *star_column;
     GtkWidget *column_editor;
     GHashTable *factory_to_column_map;
-
-    GHashTable *all_view_columns_hash;
-
-    /* Column sort hack state */
     gboolean column_header_was_clicked;
-    GQuark clicked_column_attribute_q;
 };
 
 G_DEFINE_TYPE (NautilusListView, nautilus_list_view, NAUTILUS_TYPE_LIST_BASE)
 
-
-static void on_sorter_changed (GtkSorter      *sorter,
-                               GtkSorterChange change,
-                               gpointer        user_data);
-
 static const char *default_columns_for_recent[] =
 {
     "name", "size", "recency", NULL
@@ -123,6 +117,7 @@ apply_columns_settings (NautilusListView  *self,
     g_autoptr (GList) view_columns = NULL;
     GListModel *old_view_columns;
     g_autoptr (GHashTable) visible_columns_hash = NULL;
+    g_autoptr (GHashTable) old_view_columns_hash = NULL;
     int column_i = 0;
 
     file = nautilus_files_view_get_directory_as_file (NAUTILUS_FILES_VIEW (self));
@@ -167,7 +162,25 @@ apply_columns_settings (NautilusListView  *self,
         }
     }
 
+    old_view_columns_hash = g_hash_table_new_full (g_str_hash,
+                                                   g_str_equal,
+                                                   (GDestroyNotify) g_free,
+                                                   NULL);
     old_view_columns = gtk_column_view_get_columns (self->view_ui);
+    for (guint i = 0; i < g_list_model_get_n_items (old_view_columns); i++)
+    {
+        g_autoptr (GtkColumnViewColumn) view_column = NULL;
+        GtkListItemFactory *factory;
+        NautilusColumn *nautilus_column;
+        gchar *name;
+
+        view_column = g_list_model_get_item (old_view_columns, i);
+        factory = gtk_column_view_column_get_factory (view_column);
+        nautilus_column = g_hash_table_lookup (self->factory_to_column_map, factory);
+        g_object_get (nautilus_column, "name", &name, NULL);
+        g_hash_table_insert (old_view_columns_hash, name, view_column);
+    }
+
     for (GList *l = all_columns; l != NULL; l = l->next)
     {
         g_autofree char *name = NULL;
@@ -180,7 +193,7 @@ apply_columns_settings (NautilusListView  *self,
         {
             GtkColumnViewColumn *view_column;
 
-            view_column = g_hash_table_lookup (self->all_view_columns_hash, name);
+            view_column = g_hash_table_lookup (old_view_columns_hash, name);
             if (view_column != NULL)
             {
                 view_columns = g_list_prepend (view_columns, view_column);
@@ -198,7 +211,11 @@ apply_columns_settings (NautilusListView  *self,
         view_column = g_list_model_get_item (old_view_columns, i);
         if (g_list_find (view_columns, view_column) == NULL)
         {
-            gtk_column_view_remove_column (self->view_ui, view_column);
+            gtk_column_view_column_set_visible (view_column, FALSE);
+        }
+        else
+        {
+            gtk_column_view_column_set_visible (view_column, TRUE);
         }
     }
 
@@ -229,34 +246,15 @@ real_scroll_to_item (NautilusListBase *list_base_view,
     }
 }
 
-typedef struct
-{
-    GQuark attribute;
-    NautilusListView *view;
-} SortData;
-
 static gint
 nautilus_list_view_sort (gconstpointer a,
                          gconstpointer b,
                          gpointer      user_data)
 {
-    SortData *data = user_data;
-    NautilusListView *self = data->view;
-    GQuark attribute_q = data->attribute;
+    GQuark attribute_q = GPOINTER_TO_UINT (user_data);
     NautilusFile *file_a = nautilus_view_item_get_file (NAUTILUS_VIEW_ITEM ((gpointer) a));
     NautilusFile *file_b = nautilus_view_item_get_file (NAUTILUS_VIEW_ITEM ((gpointer) b));
 
-    /* Hack: We don't know what column is being sorted on when the column
-     * headers are clicked. So let's just look at what attribute was most
-     * recently used for sorting.
-     * https://gitlab.gnome.org/GNOME/gtk/-/issues/4833 */
-    if (self->clicked_column_attribute_q == 0 && self->column_header_was_clicked)
-    {
-        self->clicked_column_attribute_q = attribute_q;
-    }
-
-    g_return_val_if_fail (file_a != NULL && file_b != NULL, GTK_ORDERING_EQUAL);
-
     /* The reversed argument is FALSE because the columnview sorter handles that
      * itself and if we don't want to reverse the reverse. The directories_first
      * argument is also FALSE for the same reason: we don't want the columnview
@@ -584,18 +582,44 @@ action_visible_columns (GSimpleAction *action,
 {
     NautilusListView *self = NAUTILUS_LIST_VIEW (user_data);
 
-    if (self->column_editor)
-    {
-        gtk_widget_show (self->column_editor);
-    }
-    else
+    if (self->column_editor == NULL)
     {
         self->column_editor = create_column_editor (self);
         g_object_add_weak_pointer (G_OBJECT (self->column_editor),
                                    (gpointer *) &self->column_editor);
-
-        gtk_widget_show (self->column_editor);
     }
+
+    gtk_window_present (GTK_WINDOW (self->column_editor));
+}
+
+static void
+on_sorter_changed (GtkSorter       *sorter,
+                   GtkSorterChange  change,
+                   gpointer         user_data)
+{
+    NautilusListView *self = NAUTILUS_LIST_VIEW (user_data);
+    GtkColumnViewSorter *column_view_sorter = GTK_COLUMN_VIEW_SORTER (sorter);
+    GActionGroup *action_group;
+    gboolean reversed;
+    GtkColumnViewColumn *primary;
+    const char *sort_text;
+
+    primary = gtk_column_view_sorter_get_primary_sort_column (column_view_sorter);
+
+    if (primary == NULL)
+    {
+        return;
+    }
+
+    reversed = gtk_column_view_sorter_get_primary_sort_order (column_view_sorter);
+    action_group = nautilus_files_view_get_action_group (NAUTILUS_FILES_VIEW (self));
+
+    sort_text = gtk_column_view_column_get_id (primary);
+
+    self->column_header_was_clicked = TRUE;
+
+    g_action_group_change_action_state (action_group, "sort",
+                                        g_variant_new ("(sb)", sort_text, reversed));
 }
 
 static void
@@ -605,31 +629,11 @@ action_sort_order_changed (GSimpleAction *action,
 {
     const gchar *target_name;
     gboolean reversed;
-    NautilusFileSortType sort_type;
     NautilusListView *self;
     GListModel *view_columns;
-    NautilusViewModel *model;
     g_autoptr (GtkColumnViewColumn) sort_column = NULL;
     GtkSorter *sorter;
 
-    /* This array makes the #NautilusFileSortType values correspond to the
-     * respective column attribute.
-     */
-    const char *attributes[] =
-    {
-        "name",
-        "size",
-        "type",
-        "date_modified",
-        "date_accessed",
-        "date_created",
-        "starred",
-        "trashed_on",
-        "search_relevance",
-        "recency",
-        NULL
-    };
-
     /* Don't resort if the action is in the same state as before */
     if (g_variant_equal (value, g_action_get_state (G_ACTION (action))))
     {
@@ -639,47 +643,27 @@ action_sort_order_changed (GSimpleAction *action,
     self = NAUTILUS_LIST_VIEW (user_data);
     g_variant_get (value, "(&sb)", &target_name, &reversed);
 
-    if (g_strcmp0 (target_name, "unknown") == 0)
-    {
-        /* Sort order has been changed without using this action. */
-        g_simple_action_set_state (action, value);
-        return;
-    }
-
-    sort_type = get_sorts_type_from_metadata_text (target_name);
-
     view_columns = gtk_column_view_get_columns (self->view_ui);
     for (guint i = 0; i < g_list_model_get_n_items (view_columns); i++)
     {
         g_autoptr (GtkColumnViewColumn) view_column = NULL;
-        GtkListItemFactory *factory;
-        NautilusColumn *nautilus_column;
-        gchar *attribute;
+        const char *attribute;
 
         view_column = g_list_model_get_item (view_columns, i);
-        factory = gtk_column_view_column_get_factory (view_column);
-        nautilus_column = g_hash_table_lookup (self->factory_to_column_map, factory);
-        if (nautilus_column == NULL)
-        {
-            continue;
-        }
-        g_object_get (nautilus_column, "attribute", &attribute, NULL);
-        if (g_strcmp0 (attributes[sort_type], attribute) == 0)
+        attribute = gtk_column_view_column_get_id (view_column);
+        if (g_strcmp0 (target_name, attribute) == 0)
         {
             sort_column = g_steal_pointer (&view_column);
             break;
         }
     }
 
-    model = nautilus_list_base_get_model (NAUTILUS_LIST_BASE (self));
-    sorter = nautilus_view_model_get_sorter (model);
+    sorter = gtk_column_view_get_sorter (self->view_ui);
 
-    /* Ask the column view to sort by column if it hasn't just done so already. */
+    /* Initiate default sort or sort from the view menu, but not when the header was clicked */
     if (!self->column_header_was_clicked)
     {
         g_signal_handlers_block_by_func (sorter, on_sorter_changed, self);
-        /* FIXME: Set NULL to stop drawing the arrow on previous sort column
-         * to workaround https://gitlab.gnome.org/GNOME/gtk/-/issues/4696 */
         gtk_column_view_sort_by_column (self->view_ui, NULL, FALSE);
         gtk_column_view_sort_by_column (self->view_ui, sort_column, reversed);
         g_signal_handlers_unblock_by_func (sorter, on_sorter_changed, self);
@@ -747,14 +731,15 @@ static void
 real_begin_loading (NautilusFilesView *files_view)
 {
     NautilusListView *self = NAUTILUS_LIST_VIEW (files_view);
+    NautilusViewModel *model;
     NautilusFile *file;
 
-    /* We need to setup the columns before chaining up */
-    update_columns_settings_from_metadata_and_preferences (self);
-
     NAUTILUS_FILES_VIEW_CLASS (nautilus_list_view_parent_class)->begin_loading (files_view);
 
-    self->clicked_column_attribute_q = 0;
+    update_columns_settings_from_metadata_and_preferences (self);
+
+    self->expand_as_a_tree = g_settings_get_boolean (nautilus_list_view_preferences,
+                                                     NAUTILUS_PREFERENCES_LIST_VIEW_USE_TREE);
 
     self->path_attribute_q = 0;
     g_clear_object (&self->file_path_base_location);
@@ -770,7 +755,21 @@ real_begin_loading (NautilusFilesView *files_view)
     {
         self->path_attribute_q = g_quark_from_string ("where");
         self->file_path_base_location = get_base_location (self);
+
+        /* Forcefully disabling tree in these special locations because this
+         * view and its model currently don't expect the same file appearing
+         * more than once.
+         *
+         * NautilusFilesView still has support for the same file being present
+         * in multiple directories (struct FileAndDirectory), so, if someone
+         * cares enough about expanding folders in these special locations:
+         * TODO: Making the model items aware of their current model instead of
+         * relying on `nautilus_file_get_parent()`. */
+        self->expand_as_a_tree = FALSE;
     }
+
+    model = nautilus_list_base_get_model (NAUTILUS_LIST_BASE (self));
+    nautilus_view_model_expand_as_a_tree (model, self->expand_as_a_tree);
 }
 
 static void
@@ -851,69 +850,7 @@ real_sort_directories_first_changed (NautilusFilesView *files_view)
 
     self->directories_first = nautilus_files_view_should_sort_directories_first (NAUTILUS_FILES_VIEW (self));
 
-    nautilus_list_base_reset_sort (NAUTILUS_LIST_BASE (self));
-}
-
-static void
-on_sorter_changed (GtkSorter       *sorter,
-                   GtkSorterChange  change,
-                   gpointer         user_data)
-{
-    NautilusListView *self = NAUTILUS_LIST_VIEW (user_data);
-    NautilusViewModel *model = nautilus_list_base_get_model (NAUTILUS_LIST_BASE (self));
-
-    /* Set the conditions to capture the sort attribute the first time that
-     * nautilus_list_view_sort() is called. */
-    self->column_header_was_clicked = TRUE;
-    self->clicked_column_attribute_q = 0;
-
-    /* If there is only one file, enforce a comparison against a dummy item, to
-     * ensure nautilus_list_view_sort() gets called at least once. */
-    if (g_list_model_get_n_items (G_LIST_MODEL (model)) == 1)
-    {
-        NautilusViewItem *item = g_list_model_get_item (G_LIST_MODEL (model), 0);
-        g_autoptr (NautilusViewItem) dummy_item = NULL;
-
-        dummy_item = nautilus_view_item_new (nautilus_view_item_get_file (item),
-                                             NAUTILUS_LIST_ICON_SIZE_SMALL);
-
-        gtk_sorter_compare (sorter, item, dummy_item);
-    }
-}
-
-static void
-on_after_sorter_changed (GtkSorter       *sorter,
-                         GtkSorterChange  change,
-                         gpointer         user_data)
-{
-    NautilusListView *self = NAUTILUS_LIST_VIEW (user_data);
-    GActionGroup *action_group = nautilus_files_view_get_action_group (NAUTILUS_FILES_VIEW (self));
-    g_autoptr (GVariant) state = NULL;
-    const gchar *new_sort_text;
-    gboolean reversed;
-    const gchar *current_sort_text;
-
-    if (!self->column_header_was_clicked || self->clicked_column_attribute_q == 0)
-    {
-        return;
-    }
-
-    state = g_action_group_get_action_state (action_group, "sort");
-    g_variant_get (state, "(&sb)", &current_sort_text, &reversed);
-
-    new_sort_text = g_quark_to_string (self->clicked_column_attribute_q);
-
-    if (g_strcmp0 (new_sort_text, current_sort_text) == 0)
-    {
-        reversed = !reversed;
-    }
-    else
-    {
-        reversed = FALSE;
-    }
-
-    g_action_group_change_action_state (action_group, "sort",
-                                        g_variant_new ("(sb)", new_sort_text, reversed));
+    nautilus_view_model_sort (nautilus_list_base_get_model (NAUTILUS_LIST_BASE (self)));
 }
 
 static guint
@@ -922,6 +859,147 @@ real_get_view_id (NautilusFilesView *files_view)
     return NAUTILUS_VIEW_LIST_ID;
 }
 
+typedef struct
+{
+    NautilusListView *self;
+    NautilusViewItem *item;
+    NautilusDirectory *directory;
+} UnloadDelayData;
+
+static void
+unload_delay_data_free (UnloadDelayData *unload_data)
+{
+    g_clear_weak_pointer (&unload_data->self);
+    g_clear_object (&unload_data->item);
+    g_clear_object (&unload_data->directory);
+
+    g_free (unload_data);
+}
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (UnloadDelayData, unload_delay_data_free)
+
+static UnloadDelayData *
+unload_delay_data_new (NautilusListView  *self,
+                       NautilusViewItem  *item,
+                       NautilusDirectory *directory)
+{
+    UnloadDelayData *unload_data;
+
+    unload_data = g_new0 (UnloadDelayData, 1);
+    g_set_weak_pointer (&unload_data->self, self);
+    g_set_object (&unload_data->item, item);
+    g_set_object (&unload_data->directory, directory);
+
+    return unload_data;
+}
+
+static gboolean
+unload_file_timeout (gpointer data)
+{
+    g_autoptr (UnloadDelayData) unload_data = data;
+    NautilusListView *self = unload_data->self;
+    NautilusViewModel *model;
+
+    if (unload_data->self == NULL)
+    {
+        return G_SOURCE_REMOVE;
+    }
+    model = nautilus_list_base_get_model (NAUTILUS_LIST_BASE (self));
+
+    for (guint i = 0; i < g_list_model_get_n_items (G_LIST_MODEL (model)); i++)
+    {
+        g_autoptr (GtkTreeListRow) row = g_list_model_get_item (G_LIST_MODEL (model), i);
+        g_autoptr (NautilusViewItem) item = gtk_tree_list_row_get_item (row);
+        if (item != NULL && item == unload_data->item)
+        {
+            if (gtk_tree_list_row_get_expanded (row))
+            {
+                /* It has been expanded again before the timeout. Do nothing. */
+                return G_SOURCE_REMOVE;
+            }
+            break;
+        }
+    }
+
+    if (nautilus_files_view_has_subdirectory (NAUTILUS_FILES_VIEW (self),
+                                              unload_data->directory))
+    {
+        nautilus_files_view_remove_subdirectory (NAUTILUS_FILES_VIEW (self),
+                                                 unload_data->directory);
+    }
+
+    /* The model holds a GListStore for every subdirectory. Empty it. */
+    nautilus_view_model_clear_subdirectory (model, unload_data->item);
+
+    return G_SOURCE_REMOVE;
+}
+
+static void
+on_subdirectory_done_loading (NautilusDirectory *directory,
+                              GtkTreeListRow    *row)
+{
+    g_autoptr (NautilusViewItem) item = NULL;
+
+    g_signal_handlers_disconnect_by_func (directory, on_subdirectory_done_loading, row);
+
+    item = NAUTILUS_VIEW_ITEM (gtk_tree_list_row_get_item (row));
+    nautilus_view_item_set_loading (item, FALSE);
+
+    if (!nautilus_directory_is_not_empty (directory))
+    {
+        GtkWidget *name_cell = nautilus_view_item_get_item_ui (item);
+        GtkTreeExpander *expander = nautilus_name_cell_get_expander (NAUTILUS_NAME_CELL (name_cell));
+
+        gtk_tree_expander_set_hide_expander (expander, TRUE);
+    }
+}
+
+static void
+on_row_expanded_changed (GObject    *gobject,
+                         GParamSpec *pspec,
+                         gpointer    user_data)
+{
+    GtkTreeListRow *row = GTK_TREE_LIST_ROW (gobject);
+    NautilusListView *self = NAUTILUS_LIST_VIEW (user_data);
+    g_autoptr (NautilusViewItem) item = NULL;
+    g_autoptr (NautilusDirectory) directory = NULL;
+    gboolean expanded;
+
+    item = NAUTILUS_VIEW_ITEM (gtk_tree_list_row_get_item (row));
+    if (item == NULL)
+    {
+        /* Row has been destroyed. */
+        return;
+    }
+
+    directory = nautilus_directory_get_for_file (nautilus_view_item_get_file (item));
+    expanded = gtk_tree_list_row_get_expanded (row);
+    if (expanded)
+    {
+        if (!nautilus_files_view_has_subdirectory (NAUTILUS_FILES_VIEW (self), directory))
+        {
+            nautilus_files_view_add_subdirectory (NAUTILUS_FILES_VIEW (self), directory);
+        }
+        if (!nautilus_directory_are_all_files_seen (directory))
+        {
+            nautilus_view_item_set_loading (item, TRUE);
+
+            g_signal_connect_object (directory,
+                                     "done-loading",
+                                     G_CALLBACK (on_subdirectory_done_loading),
+                                     row,
+                                     0);
+        }
+    }
+    else
+    {
+        nautilus_view_item_set_loading (item, FALSE);
+        g_timeout_add_seconds (COLLAPSE_TO_UNLOAD_DELAY,
+                               unload_file_timeout,
+                               unload_delay_data_new (self, item, directory));
+    }
+}
+
 static void
 on_item_click_released_workaround (GtkGestureClick *gesture,
                                    gint             n_press,
@@ -930,7 +1008,7 @@ on_item_click_released_workaround (GtkGestureClick *gesture,
                                    gpointer         user_data)
 {
     NautilusViewCell *cell = user_data;
-    g_autoptr (NautilusListView) self = NAUTILUS_LIST_VIEW (nautilus_view_cell_get_view (cell));
+    NautilusListView *self = NAUTILUS_LIST_VIEW (nautilus_view_cell_get_view (cell));
     GdkModifierType modifiers;
 
     modifiers = gtk_event_controller_get_current_event_state (GTK_EVENT_CONTROLLER (gesture));
@@ -974,6 +1052,72 @@ setup_selection_click_workaround (NautilusViewCell *cell)
     g_signal_connect (controller, "released", G_CALLBACK (on_item_click_released_workaround), cell);
 }
 
+static gboolean
+tree_expander_shortcut_cb (GtkWidget *widget,
+                           GVariant  *args,
+                           gpointer   user_data)
+{
+    NautilusListView *self = NAUTILUS_LIST_VIEW (widget);
+    GtkWidget *child;
+    g_autoptr (NautilusViewItem) item = NULL;
+    GtkTreeExpander *expander;
+    GtkTreeListRow *row;
+    GtkTextDirection direction = gtk_widget_get_direction (widget);
+    char *action;
+    guint keyval = GPOINTER_TO_INT (user_data);
+
+    /* Hack to find the focus item. */
+    child = gtk_root_get_focus (gtk_widget_get_root (widget));
+    while (child != NULL && !NAUTILUS_IS_VIEW_CELL (child))
+    {
+        child = gtk_widget_get_first_child (child);
+    }
+
+    if (!NAUTILUS_IS_VIEW_CELL (child))
+    {
+        return FALSE;
+    }
+
+    /* The name cell might not be the first column (the user may have changed
+     * column order), but the expander is always on the name cell. */
+    item = nautilus_view_cell_get_item (NAUTILUS_VIEW_CELL (child));
+    expander = nautilus_name_cell_get_expander (NAUTILUS_NAME_CELL (nautilus_view_item_get_item_ui (item)));
+    row = gtk_tree_expander_get_list_row (expander);
+    /* End of hack. */
+
+    if ((keyval == GDK_KEY_Right && direction == GTK_TEXT_DIR_LTR) ||
+        (keyval == GDK_KEY_Left && direction == GTK_TEXT_DIR_RTL))
+    {
+        action = "listitem.expand";
+    }
+    else
+    {
+        action = "listitem.collapse";
+    }
+
+    if (!gtk_tree_list_row_get_expanded (row) &&
+        g_strcmp0 (action, "listitem.collapse") == 0)
+    {
+        g_autoptr (GtkTreeListRow) parent = gtk_tree_list_row_get_parent (row);
+
+        if (parent != NULL)
+        {
+            g_autoptr (NautilusViewItem) parent_item = NULL;
+
+            parent_item = NAUTILUS_VIEW_ITEM (gtk_tree_list_row_get_item (parent));
+            g_return_val_if_fail (parent_item != NULL, FALSE);
+
+            set_focus_item (NAUTILUS_LIST_BASE (self), parent_item);
+        }
+    }
+    else
+    {
+        gtk_widget_activate_action (GTK_WIDGET (expander), action, NULL);
+    }
+
+    return TRUE;
+}
+
 static void
 setup_name_cell (GtkSignalListItemFactory *factory,
                  GtkListItem              *listitem,
@@ -984,6 +1128,7 @@ setup_name_cell (GtkSignalListItemFactory *factory,
 
     cell = nautilus_name_cell_new (NAUTILUS_LIST_BASE (self));
     setup_cell_common (listitem, cell);
+    setup_cell_hover_inner_target (cell, nautilus_name_cell_get_content (NAUTILUS_NAME_CELL (cell)));
 
     nautilus_name_cell_set_path (NAUTILUS_NAME_CELL (cell),
                                  self->path_attribute_q,
@@ -994,6 +1139,47 @@ setup_name_cell (GtkSignalListItemFactory *factory,
     }
 
     setup_selection_click_workaround (cell);
+
+    if (self->expand_as_a_tree)
+    {
+        GtkTreeExpander *expander;
+
+        expander = nautilus_name_cell_get_expander (NAUTILUS_NAME_CELL (cell));
+        gtk_tree_expander_set_indent_for_icon (expander, TRUE);
+        g_object_bind_property (listitem, "item",
+                                expander, "list-row",
+                                G_BINDING_SYNC_CREATE);
+    }
+}
+
+static void
+on_n_items_notify (GObject    *object,
+                   GParamSpec *pspec,
+                   gpointer    user_data)
+{
+    GListModel *model = G_LIST_MODEL (object);
+    GtkTreeExpander *expander = GTK_TREE_EXPANDER (user_data);
+    guint n_items = g_list_model_get_n_items (model);
+
+    gtk_tree_expander_set_hide_expander (expander, n_items == 0);
+}
+
+static void
+on_row_children_changed (GObject    *gobject,
+                         GParamSpec *pspec,
+                         gpointer    user_data)
+{
+    GtkTreeExpander *expander = user_data;
+    GListModel *model = gtk_tree_list_row_get_children (GTK_TREE_LIST_ROW (gobject));
+
+    if (model == NULL)
+    {
+        return;
+    }
+
+    g_signal_connect_object (model, "notify::n-items",
+                             G_CALLBACK (on_n_items_notify), expander,
+                             0);
 }
 
 static void
@@ -1002,10 +1188,11 @@ bind_name_cell (GtkSignalListItemFactory *factory,
                 gpointer                  user_data)
 {
     GtkWidget *cell;
-    NautilusViewItem *item;
+    NautilusListView *self = user_data;
+    g_autoptr (NautilusViewItem) item = NULL;
 
     cell = gtk_list_item_get_child (listitem);
-    item = NAUTILUS_VIEW_ITEM (gtk_list_item_get_item (listitem));
+    item = listitem_get_view_item (listitem);
 
     nautilus_view_item_set_item_ui (item, gtk_list_item_get_child (listitem));
 
@@ -1022,6 +1209,21 @@ bind_name_cell (GtkSignalListItemFactory *factory,
                                         GTK_ACCESSIBLE_RELATION_LABELLED_BY, cell, NULL,
                                         -1);
     }
+
+    if (self->expand_as_a_tree)
+    {
+        GtkTreeExpander *expander = nautilus_name_cell_get_expander (NAUTILUS_NAME_CELL (cell));
+        GtkTreeListRow *row = GTK_TREE_LIST_ROW (gtk_list_item_get_item (listitem));
+
+        g_signal_connect_object (row,
+                                 "notify::expanded",
+                                 G_CALLBACK (on_row_expanded_changed),
+                                 self, 0);
+        g_signal_connect_object (row,
+                                 "notify::children",
+                                 G_CALLBACK (on_row_children_changed),
+                                 expander, 0);
+    }
 }
 
 static void
@@ -1029,12 +1231,28 @@ unbind_name_cell (GtkSignalListItemFactory *factory,
                   GtkListItem              *listitem,
                   gpointer                  user_data)
 {
-    NautilusViewItem *item;
+    NautilusListView *self = user_data;
+    g_autoptr (NautilusViewItem) item = NULL;
 
-    item = NAUTILUS_VIEW_ITEM (gtk_list_item_get_item (listitem));
+    item = listitem_get_view_item (listitem);
+    if (item == NULL)
+    {
+        /* The row is gone */
+        return;
+    }
     g_return_if_fail (NAUTILUS_IS_VIEW_ITEM (item));
 
     nautilus_view_item_set_item_ui (item, NULL);
+
+    if (self->expand_as_a_tree)
+    {
+        g_signal_handlers_disconnect_by_func (gtk_list_item_get_item (listitem),
+                                              on_row_expanded_changed,
+                                              self);
+        g_signal_handlers_disconnect_by_func (gtk_list_item_get_item (listitem),
+                                              on_row_children_changed,
+                                              self);
+    }
 }
 
 static void
@@ -1046,6 +1264,7 @@ setup_star_cell (GtkSignalListItemFactory *factory,
 
     cell = nautilus_star_cell_new (NAUTILUS_LIST_BASE (user_data));
     setup_cell_common (listitem, cell);
+    setup_cell_hover (cell);
     setup_selection_click_workaround (cell);
 }
 
@@ -1062,6 +1281,7 @@ setup_label_cell (GtkSignalListItemFactory *factory,
 
     cell = nautilus_label_cell_new (NAUTILUS_LIST_BASE (user_data), nautilus_column);
     setup_cell_common (listitem, cell);
+    setup_cell_hover (cell);
     setup_selection_click_workaround (cell);
 }
 
@@ -1077,15 +1297,10 @@ setup_view_columns (NautilusListView *self)
                                                          g_direct_equal,
                                                          NULL,
                                                          g_object_unref);
-    self->all_view_columns_hash = g_hash_table_new_full (g_str_hash,
-                                                         g_str_equal,
-                                                         (GDestroyNotify) g_free,
-                                                         g_object_unref);
 
     for (GList *l = nautilus_columns; l != NULL; l = l->next)
     {
         NautilusColumn *nautilus_column = NAUTILUS_COLUMN (l->data);
-        SortData *data;
         g_autofree gchar *name = NULL;
         g_autofree gchar *label = NULL;
         GQuark attribute_q = 0;
@@ -1100,13 +1315,9 @@ setup_view_columns (NautilusListView *self)
                       "default-sort-order", &sort_order,
                       NULL);
 
-        data = g_new0 (SortData, 1);
-        data->attribute = attribute_q;
-        data->view = self;
-
         sorter = gtk_custom_sorter_new (nautilus_list_view_sort,
-                                        data,
-                                        g_free);
+                                        GUINT_TO_POINTER (attribute_q),
+                                        NULL);
 
         factory = gtk_signal_list_item_factory_new ();
         view_column = gtk_column_view_column_new (NULL, factory);
@@ -1137,22 +1348,34 @@ setup_view_columns (NautilusListView *self)
             g_signal_connect (factory, "setup", G_CALLBACK (setup_label_cell), self);
         }
 
+        gtk_column_view_append_column (self->view_ui, view_column);
+        gtk_column_view_column_set_id (view_column, name);
+
         g_hash_table_insert (self->factory_to_column_map,
                              factory,
                              g_object_ref (nautilus_column));
-        g_hash_table_insert (self->all_view_columns_hash,
-                             g_steal_pointer (&name),
-                             g_steal_pointer (&view_column));
     }
 }
 
+static void
+nautilus_list_view_reload (NautilusListView *self)
+{
+    NautilusWindowSlot *slot;
+
+    slot = nautilus_files_view_get_nautilus_window_slot (NAUTILUS_FILES_VIEW (self));
+    nautilus_window_slot_queue_reload (slot);
+}
+
 static void
 nautilus_list_view_init (NautilusListView *self)
 {
     NautilusViewModel *model;
     GtkWidget *content_widget;
+    GtkSorter *column_view_sorter;
     g_autoptr (GtkCustomSorter) directories_sorter = NULL;
     g_autoptr (GtkMultiSorter) sorter = NULL;
+    GtkEventController *controller;
+    GtkShortcut *shortcut;
 
     gtk_widget_add_css_class (GTK_WIDGET (self), "nautilus-list-view");
 
@@ -1166,6 +1389,11 @@ nautilus_list_view_init (NautilusListView *self)
                              G_CALLBACK (update_columns_settings_from_metadata_and_preferences),
                              self,
                              G_CONNECT_SWAPPED);
+    g_signal_connect_object (nautilus_list_view_preferences,
+                             "changed::" NAUTILUS_PREFERENCES_LIST_VIEW_USE_TREE,
+                             G_CALLBACK (nautilus_list_view_reload),
+                             self,
+                             G_CONNECT_SWAPPED);
 
     content_widget = nautilus_files_view_get_content_widget (NAUTILUS_FILES_VIEW (self));
 
@@ -1178,11 +1406,11 @@ nautilus_list_view_init (NautilusListView *self)
     directories_sorter = gtk_custom_sorter_new (sort_directories_func, &self->directories_first, NULL);
 
     sorter = gtk_multi_sorter_new ();
+    column_view_sorter = gtk_column_view_get_sorter (self->view_ui);
     gtk_multi_sorter_append (sorter, g_object_ref (GTK_SORTER (directories_sorter)));
-    gtk_multi_sorter_append (sorter, g_object_ref (gtk_column_view_get_sorter (self->view_ui)));
-    g_signal_connect_object (sorter, "changed", G_CALLBACK (on_sorter_changed), self, 0);
-    g_signal_connect_object (sorter, "changed", G_CALLBACK (on_after_sorter_changed), self, G_CONNECT_AFTER);
+    gtk_multi_sorter_append (sorter, g_object_ref (column_view_sorter));
 
+    g_signal_connect (column_view_sorter, "changed", G_CALLBACK (on_sorter_changed), self);
     model = nautilus_list_base_get_model (NAUTILUS_LIST_BASE (self));
     nautilus_view_model_set_sorter (model, GTK_SORTER (sorter));
 
@@ -1198,6 +1426,18 @@ nautilus_list_view_init (NautilusListView *self)
     self->zoom_level = get_default_zoom_level ();
     g_action_group_change_action_state (nautilus_files_view_get_action_group (NAUTILUS_FILES_VIEW (self)),
                                         "zoom-to-level", g_variant_new_int32 (self->zoom_level));
+
+    /* Set up tree expand/collapse shortcuts in capture phase otherwise they
+     * would be handled by GtkListBase's cursor movement shortcuts. */
+    controller = gtk_shortcut_controller_new ();
+    gtk_widget_add_controller (GTK_WIDGET (self), controller);
+    gtk_event_controller_set_propagation_phase (controller, GTK_PHASE_CAPTURE);
+    shortcut = gtk_shortcut_new (gtk_keyval_trigger_new (GDK_KEY_Left, 0),
+                                 gtk_callback_action_new (tree_expander_shortcut_cb, GINT_TO_POINTER (GDK_KEY_Left), NULL));
+    gtk_shortcut_controller_add_shortcut (GTK_SHORTCUT_CONTROLLER (controller), shortcut);
+    shortcut = gtk_shortcut_new (gtk_keyval_trigger_new (GDK_KEY_Right, 0),
+                                 gtk_callback_action_new (tree_expander_shortcut_cb, GINT_TO_POINTER (GDK_KEY_Right), NULL));
+    gtk_shortcut_controller_add_shortcut (GTK_SHORTCUT_CONTROLLER (controller), shortcut);
 }
 
 static void
@@ -1209,9 +1449,17 @@ nautilus_list_view_dispose (GObject *object)
     model = nautilus_list_base_get_model (NAUTILUS_LIST_BASE (self));
     nautilus_view_model_set_sorter (model, NULL);
 
+    g_signal_handlers_disconnect_by_func (nautilus_list_view_preferences,
+                                          update_columns_settings_from_metadata_and_preferences,
+                                          self);
+    g_signal_handlers_disconnect_by_func (nautilus_list_view_preferences,
+                                          nautilus_list_view_reload,
+                                          self);
+
     g_clear_object (&self->file_path_base_location);
     g_clear_pointer (&self->factory_to_column_map, g_hash_table_destroy);
-    g_clear_pointer (&self->all_view_columns_hash, g_hash_table_destroy);
+
+    g_signal_handlers_disconnect_by_func (gtk_column_view_get_sorter (self->view_ui), on_sorter_changed, self);
 
     G_OBJECT_CLASS (nautilus_list_view_parent_class)->dispose (object);
 }
