@@ -32,7 +32,6 @@ struct _NautilusProgressInfoWidgetPrivate
     GtkWidget *details;     /* GtkLabel */
     GtkWidget *progress_bar;
     GtkWidget *button;
-    GtkWidget *image;
 };
 
 enum
@@ -49,9 +48,7 @@ G_DEFINE_TYPE_WITH_PRIVATE (NautilusProgressInfoWidget, nautilus_progress_info_w
 static void
 info_finished (NautilusProgressInfoWidget *self)
 {
-    gtk_image_set_from_icon_name (GTK_IMAGE (self->priv->image),
-                                  "object-select-symbolic",
-                                  GTK_ICON_SIZE_BUTTON);
+    gtk_button_set_icon_name (GTK_BUTTON (self->priv->button), "object-select-symbolic");
     gtk_widget_set_sensitive (self->priv->button, FALSE);
 }
 
@@ -127,9 +124,7 @@ nautilus_progress_info_widget_constructed (GObject *obj)
 
     if (nautilus_progress_info_get_is_finished (self->priv->info))
     {
-        gtk_image_set_from_icon_name (GTK_IMAGE (self->priv->image),
-                                      "object-select-symbolic",
-                                      GTK_ICON_SIZE_BUTTON);
+        gtk_button_set_icon_name (GTK_BUTTON (self->priv->button), "object-select-symbolic");
     }
 
     gtk_widget_set_sensitive (self->priv->button,
@@ -217,7 +212,6 @@ nautilus_progress_info_widget_class_init (NautilusProgressInfoWidgetClass *klass
     gtk_widget_class_bind_template_child_private (widget_class, NautilusProgressInfoWidget, details);
     gtk_widget_class_bind_template_child_private (widget_class, NautilusProgressInfoWidget, progress_bar);
     gtk_widget_class_bind_template_child_private (widget_class, NautilusProgressInfoWidget, button);
-    gtk_widget_class_bind_template_child_private (widget_class, NautilusProgressInfoWidget, image);
 }
 
 GtkWidget *
