@@ -302,8 +302,7 @@ visit_directory (GFile            *dir,
     enumerator = g_file_enumerate_children (dir,
                                             data->mime_types->len > 0 ?
                                             STD_ATTRIBUTES ","
-                                            G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
-                                            G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE
+                                            G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE
                                             :
                                             STD_ATTRIBUTES
                                             ,
@@ -348,15 +347,9 @@ visit_directory (GFile            *dir,
         {
             mime_type = g_file_info_get_attribute_string (info,
                                                           G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
-            if (mime_type == NULL)
-            {
-                mime_type = g_file_info_get_attribute_string (info,
-                                                              G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE);
-            }
-
             found = FALSE;
 
-            for (gint i = 0; i < data->mime_types->len; i++)
+            for (gint i = 0; mime_type != NULL && i < data->mime_types->len; i++)
             {
                 if (g_content_type_is_a (mime_type, g_ptr_array_index (data->mime_types, i)))
                 {
